/// <reference types="node" />
import { APIGatewayProxyEvent, APIGatewayProxyEventV2, APIGatewayProxyResult, Context as LambdaContext } from 'aws-lambda';
import { FileUploadOptions, ApolloServerBase, GraphQLOptions } from 'apollo-server-core';
import { IncomingHttpHeaders } from 'http';
import { Readable } from 'stream';
export declare type APIGatewayProxyEventV1OrV2 = APIGatewayProxyEvent | APIGatewayProxyEventV2;
export interface CreateHandlerOptions<EventT extends APIGatewayProxyEventV1OrV2 = APIGatewayProxyEventV1OrV2> {
    cors?: {
        origin?: boolean | string | string[];
        methods?: string | string[];
        allowedHeaders?: string | string[];
        exposedHeaders?: string | string[];
        credentials?: boolean;
        maxAge?: number;
    };
    uploadsConfig?: FileUploadOptions;
    onHealthCheck?: (req: EventT) => Promise<any>;
}
export declare class FileUploadRequest extends Readable {
    headers: IncomingHttpHeaders;
}
export declare class ApolloServer<EventT extends APIGatewayProxyEventV1OrV2 = APIGatewayProxyEventV1OrV2> extends ApolloServerBase {
    protected serverlessFramework(): boolean;
    protected supportsUploads(): boolean;
    createGraphQLServerOptions(event: EventT, context: LambdaContext): Promise<GraphQLOptions>;
    createHandler({ cors, onHealthCheck }?: CreateHandlerOptions<EventT>): (event: EventT, context: LambdaContext, callback: import("aws-lambda").Callback<APIGatewayProxyResult> | undefined) => void | Promise<APIGatewayProxyResult>;
}
//# sourceMappingURL=ApolloServer.d.ts.map